<?php

namespace Slack\Blockkit;

use Illuminate\Support\Str;
use Slack\BlockKit;

/**
 * This class creates a slack Modal Response
 */
class Modal extends BlockKit
{
	protected $blocks;
	private $action = NULL;

	public function __construct(string $title)
	{
		parent::__construct();

		$this->blocks = collect();

		$this->_data->put('type','modal');
		$this->_data->put('title',$this->text(Str::limit($title,24),'plain_text'));
	}

	public function action(string $action)
	{
		$this->action = $action;
	}

	/**
	 * The data that will be returned when converted to JSON.
	 */
	public function jsonSerialize()
	{
		if ($this->blocks->count())
			$this->_data->put('blocks',$this->blocks);

		switch ($this->action) {
			case 'clear':
				return ['response_action'=>'clear'];

			case 'update':
				return ['response_action'=>'update','view'=>$this->_data];

			default:
				return $this->_data;
		}
	}

	/**
	 * Add a block to the modal
	 *
	 * @param Block $block
	 * @return $this
	 */
	public function addBlock(Block $block): self
	{
		$this->blocks->push($block);

		return $this;
	}

	public function callback(string $id): self
	{
		$this->_data->put('callback_id',$id);

		return $this;
	}

	public function close(string $text='Cancel'): self
	{
		$this->_data->put('close',
			[
				'type'=>'plain_text',
				'text'=>$text,
				'emoji'=>true,
			]);

		return $this;
	}

	public function meta(string $id): self
	{
		$this->_data->put('private_metadata',$id);

		return $this;
	}

	public function notifyClose(): self
	{
		$this->_data->put('notify_on_close',TRUE);

		return $this;
	}

	public function private(array $data): self
	{
		$this->_data->put('private_metadata',json_encode($data));

		return $this;
	}

	public function submit(string $text='Submit'): self
	{
		$this->_data->put('submit',
			[
				'type'=>'plain_text',
				'text'=>$text,
				'emoji'=>true,
			]);

		return $this;
	}
}
