<?php

namespace Slack\Client;

use Devristo\Phpws\Client\WebSocket;
use Devristo\Phpws\Messaging\WebSocketMessageInterface;
use Evenement\EventEmitterTrait;
use GuzzleHttp\ClientInterface;
use Illuminate\Http\Client\ConnectionException;
use Illuminate\Support\Facades\Log;
use React\EventLoop\LoopInterface;
use React\Promise\Deferred;
use React\Promise;

class SocketMode extends API
{
	use EventEmitterTrait;

	private bool $connected;
	private WebSocket $websocket;

	/**
	 * @var \Zend\Log\Logger Logger for this client
	 */
	protected $logger = null;
	private array $pendingMessages;

	public function __construct(LoopInterface $loop, ClientInterface $httpClient = null) {
		parent::__construct($loop, $httpClient);

		$this->logger = new \Zend\Log\Logger();
		$this->logger->addWriter(new \Zend\Log\Writer\Stream('php://stderr'));
	}

	/**
	 * Connects to the real-time messaging server.
	 *
	 * @return \React\Promise\PromiseInterface
	 */
	public function connect()
	{
		$deferred = new Deferred;

		// Request a real-time connection...
		$this->apiCall('apps.connections.open')
			// then connect to the socket...
			->then(function (Payload $response) {
				/*
				$responseData = $response->getData();
				// get the team info
				$this->team = new Team($this, $responseData['team']);

				// Populate self user.
				$this->users[$responseData['self']['id']] = new User($this, $responseData['self']);

				// populate list of users
				foreach ($responseData['users'] as $data) {
					$this->users[$data['id']] = new User($this, $data);
				}

				// populate list of channels
				foreach ($responseData['channels'] as $data) {
					$this->channels[$data['id']] = new Channel($this, $data);
				}

				// populate list of groups
				foreach ($responseData['groups'] as $data) {
					$this->groups[$data['id']] = new Group($this, $data);
				}

				// populate list of dms
				foreach ($responseData['ims'] as $data) {
					$this->dms[$data['id']] = new DirectMessageChannel($this, $data);
				}

				// populate list of bots
				foreach ($responseData['bots'] as $data) {
					$this->bots[$data['id']] = new Bot($this, $data);
				}
				*/

				// initiate the websocket connection
				// write PHPWS things to the existing logger
				$this->websocket = new WebSocket($response['url'].'&debug_reconnects=true', $this->loop, $this->logger);
				$this->websocket->on('message', function ($message) {
					Log::debug('Calling onMessage for',['m'=>serialize($message)]);
					$this->onMessage($message);
				});

				return $this->websocket->open();

			}, function($exception) use ($deferred) {
				// if connection was not successful
				$deferred->reject(new ConnectionException(
					'Could not connect to Slack API: '.$exception->getMessage(),
					$exception->getCode()
				));
			})

			// then wait for the connection to be ready.
			->then(function () use ($deferred) {
				$this->once('hello', function () use ($deferred) {
					$deferred->resolve();
				});

				$this->once('error', function ($data) use ($deferred) {
					$deferred->reject(new ConnectionException(
						'Could not connect to WebSocket: '.$data['error']['msg'],
						$data['error']['code']));
				});
			});

		return $deferred->promise();
	}

	/**
	 * Disconnects the client.
	 */
	public function disconnect()
	{
		if (! $this->connected) {
			return Promise\reject(new ConnectionException('Client not connected. Did you forget to call `connect()`?'));
		}

		$this->websocket->close();
		$this->connected = FALSE;
	}

	/**
	 * Returns whether the client is connected.
	 *
	 * @return bool
	 */
	public function isConnected()
	{
		return $this->connected;
	}

	/**
	 * Handles incoming websocket messages, parses them, and emits them as remote events.
	 *
	 * @param WebSocketMessageInterface $messageRaw A websocket message.
	 */
	private function onMessage(WebSocketMessageInterface $message)
	{
		Log::debug('+ Start',['m'=>__METHOD__]);

		// parse the message and get the event name
		$payload = Payload::fromJson($message->getData());

		if (isset($payload['type'])) {
			$this->emit('_internal_message', [$payload['type'], $payload]);
			switch ($payload['type']) {
				case 'hello':
					$this->connected = TRUE;
					break;

					/*
				case 'team_rename':
					$this->team->data['name'] = $payload['name'];
					break;

				case 'team_domain_change':
					$this->team->data['domain'] = $payload['domain'];
					break;

				case 'channel_joined':
					$channel = new Channel($this, $payload['channel']);
					$this->channels[$channel->getId()] = $channel;
					break;

				case 'channel_created':
					$this->getChannelById($payload['channel']['id'])->then(function (Channel $channel) {
						$this->channels[$channel->getId()] = $channel;
					});
					break;

				case 'channel_deleted':
					unset($this->channels[$payload['channel']]);
					break;

				case 'channel_rename':
					$this->channels[$payload['channel']['id']]->data['name']
						= $payload['channel']['name'];
					break;

				case 'channel_archive':
					$this->channels[$payload['channel']]->data['is_archived'] = true;
					break;

				case 'channel_unarchive':
					$this->channels[$payload['channel']]->data['is_archived'] = false;
					break;

				case 'group_joined':
					$group = new Group($this, $payload['channel']);
					$this->groups[$group->getId()] = $group;
					break;

				case 'group_rename':
					$this->groups[$payload['group']['id']]->data['name']
						= $payload['channel']['name'];
					break;

				case 'group_archive':
					$this->groups[$payload['group']['id']]->data['is_archived'] = true;
					break;

				case 'group_unarchive':
					$this->groups[$payload['group']['id']]->data['is_archived'] = false;
					break;

				case 'im_created':
					$dm = new DirectMessageChannel($this, $payload['channel']);
					$this->dms[$dm->getId()] = $dm;
					break;

				case 'bot_added':
					$bot = new Bot($this, $payload['bot']);
					$this->bots[$bot->getId()] = $bot;
					break;

				case 'bot_changed':
					$bot = new Bot($this, $payload['bot']);
					$this->bots[$bot->getId()] = $bot;
					break;

				case 'team_join':
					$user = new User($this, $payload['user']);
					$this->users[$user->getId()] = $user;
					break;

				case 'user_change':
					$user = new User($this, $payload['user']);
					$this->users[$user->getId()] = $user;
					break;
					*/
				default:
					Log::debug(sprintf('Unhandled type [%s]',$payload['type']),['m'=>__METHOD__,'p'=>$payload]);
			}

			// emit an event with the attached json
			$this->emit($payload['type'], [$payload]);
		}

		if (isset($payload['envelope_id'])) {
			// @acknowledge the event
			$this->websocket->send(json_encode(['envelope_id'=>$payload['envelope_id']]));
			Log::debug(sprintf('Responded to event [%s] for (%s)',$payload['envelope_id'],$payload['type']),['m'=>__METHOD__]);
		}

		if (! isset($payload['type']) || $payload['type'] == 'pong') {
			// If reply_to is set, then it is a server confirmation for a previously
			// sent message
			if (isset($payload['reply_to'])) {
				if (isset($this->pendingMessages[$payload['reply_to']])) {
					$deferred = $this->pendingMessages[$payload['reply_to']];

					// Resolve or reject the promise that was waiting for the reply.
					if (isset($payload['ok']) && $payload['ok'] === true || $payload['type'] == 'pong') {
						$deferred->resolve();

					} else {
						$deferred->reject($payload['error']);
					}

					unset($this->pendingMessages[$payload['reply_to']]);
				}
			}
		}

		Log::debug('= End',['m'=>__METHOD__]);
	}
}
