<?php

namespace Slack\Command;

use Illuminate\Http\Request;
use Illuminate\Support\Arr;
use Illuminate\Support\Facades\App;
use Illuminate\Support\Facades\Log;

class Factory {
	private const LOGKEY = 'SCf';

	/**
	 * @var array event type to event class mapping
	 */
	public const map = [
		'ask'=>Watson::class,
		'ate'=>Ask::class,
		'help'=>Help::class,
		'goto'=>Link::class,
		'leaders'=>Leaders::class,
		'products'=>Products::class,
		'review'=>Review::class,
		'wc'=>WatsonCollection::class,
	];

	/**
	 * Returns new event instance
	 *
	 * @param string  $type
	 * @param Request $request
	 * @return Base
	 */
	public static function create(string $type,Request $request)
	{
		$class = Arr::get(self::map,$type,Unknown::class);
		Log::debug(sprintf('%s:Working out Slash Command Class for [%s] as [%s]',static::LOGKEY,$type,$class),['m'=>__METHOD__]);

		if (App::environment() == 'dev')
			file_put_contents('/tmp/command.'.$type,print_r(json_decode(json_encode($request->all())),TRUE));

		return new $class($request);
	}

	public static function make(Request $request): Base
	{
		$data = json_decode(json_encode($request->all()));
		$command = preg_replace('/^([a-z]+)(\s?.*)/','$1',$data->text);

		return self::create($command ?: 'help',$request);
	}
}
