<?php

namespace Slack\Interactive;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Slack\Base as SlackBase;

abstract class Base extends SlackBase
{
	// Does the event respond with a reply to the HTTP request, or via a post with a trigger
	public $respondNow = FALSE;

	// When retrieving multiple action values, this is the index we are retrieving.
	protected $index = 0;

	public function __construct(Request $request)
	{
		Log::info(sprintf('SIb:Slack INTERACTIVE MESSAGE Initialised [%s]',get_class($this)),['m'=>__METHOD__]);

		// Our data is in a payload value
		$this->_data = json_decode($request->input('payload'));
	}

	/**
	 * Enable getting values for keys in the response
	 *
	 * @note: This method is limited to certain values to ensure integrity reasons
	 * @note: Classes should return:
	 *    + channel_id,
	 *    + team_id,
	 *    + ts,
	 *    + user_id
	 * @param string $key
	 * @return mixed|object
	 */
	public function __get(string $key)
	{
		switch ($key) {
			case 'enterprise_id':
				return object_get($this->_data,'team.enterprise_id');
			case 'team_id':
				return object_get($this->_data,'team.id');
			case 'user_id':
				return object_get($this->_data,'user.id');

			case 'callback_id':
			case 'trigger_id':
			case 'type':
				return object_get($this->_data,$key);
		}
	}

	/**
	 * Enable updating the index to actions with $event->index = <id>
	 *
	 * @param $key
	 * @param $value
	 */
	public function __set($key,$value)
	{
		if ($key == 'index')
			$this->{$key} = $value;
	}
}
